'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _through = require('through2');

var _through2 = _interopRequireDefault(_through);

var _gulpUtil = require('gulp-util');

var _gulpUtil2 = _interopRequireDefault(_gulpUtil);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _babylon = require('babylon');

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PluginError = _gulpUtil2.default.PluginError;
var PLUGIN_NAME = 'gulp-react-css-usage';
var cssClassRegex = /\.(-?[_a-zA-Z]+[_a-zA-Z0-9-]*)(?![^\{]*\})/gm;

var error = undefined;

var getAllClassNamesFromCSSFile = function getAllClassNamesFromCSSFile(cssFile) {
	var contents = cssFile.contents.toString();
	var classNames = {};
	var matches = cssClassRegex.exec(contents);
	while (matches != null) {
		var className = matches[1];
		classNames[className] = className;
		matches = cssClassRegex.exec(contents);
	}

	return classNames;
};

var makeDiff = function makeDiff(cssClasses, jsxClasses) {
	var needless = [];
	Object.keys(cssClasses).forEach(function (className) {
		if (!jsxClasses[className]) {
			needless.push('.' + className);
		}
	});

	return needless;
};

var printNeedlessClassList = function printNeedlessClassList(list) {
	_gulpUtil2.default.log('');
	_gulpUtil2.default.log(_gulpUtil2.default.colors.yellow('gulp-react-css-usage report: The following class names are not in use'));
	list.forEach(function (clazz) {
		return _gulpUtil2.default.log(clazz);
	});
	_gulpUtil2.default.log('');
};

var parseAndExtractJsxClassNames = function parseAndExtractJsxClassNames(jsxFileContents) {
	var babylonPlugins = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

	var jsxClassNames = {};
	var plugins = ['jsx', 'classProperties'];
	if (babylonPlugins.length) {
		plugins = plugins.concat(babylonPlugins);
	}

	// use babylon.parse and then babel traverse for dynamic class names on the jsx code.
	// might come up with a bit more strings but the needless stuff are not here anyway.
	var ast = (0, _babylon.parse)(jsxFileContents, { sourceType: 'module', plugins: plugins });
	(0, _babelTraverse2.default)(ast, {
		enter: function enter(path) {
			var _path$node = path.node;
			var type = _path$node.type;
			var value = _path$node.value;

			if (type === 'StringLiteral') {
				var classNames = value.split(' ');
				classNames.forEach(function (className) {
					return jsxClassNames[className] = className;
				});
			}
		}
	});

	return jsxClassNames;
};

var gulpReactCssUsage = function gulpReactCssUsage() {
	var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
	var cssFilePath = options.css;
	var babylon = options.babylon;

	if (!cssFilePath) {
		throw new PluginError(PLUGIN_NAME, 'Missing css field!');
	}
	if (typeof cssFilePath !== 'string') {
		throw new PluginError(PLUGIN_NAME, 'css field must be a string!');
	}

	var cssFile = new _gulpUtil2.default.File({ path: cssFilePath, contents: _fs2.default.readFileSync(cssFilePath) });
	var cssClasses = getAllClassNamesFromCSSFile(cssFile);
	var fileBuffer = undefined;
	var allClassNames = {};
	var transformers = function transformers(file, enc, cb) {
		var currentJsxClassNames = undefined;
		if (file.isNull()) {
			return cb(error, file);
		}
		if (file.isBuffer()) {
			currentJsxClassNames = parseAndExtractJsxClassNames(file.contents.toString(), babylon);
			Object.assign(allClassNames, currentJsxClassNames);
			cb(error, file);
		}
		if (file.isStream()) {
			if (enc !== 'utf8') {
				file.contents.setEncoding('utf8');
			}
			if (!fileBuffer) {
				fileBuffer = new Buffer([]); // utf8 by default
			}

			file.contents.on('data', function (chunk) {
				return fileBuffer = Buffer.concat([new Buffer(chunk), fileBuffer]);
			});
			file.contents.on('end', function () {
				currentJsxClassNames = parseAndExtractJsxClassNames(fileBuffer.toString(), babylon);
				Object.assign(allClassNames, currentJsxClassNames);
				fileBuffer = undefined;
				cb(error, file);
			});
		}
	};

	var flush = function flush(cb) {
		var needless = makeDiff(cssClasses, allClassNames);
		printNeedlessClassList(needless);
		cb();
	};

	return _through2.default.obj({}, transformers, flush);
};

// Object.assign() polyfill
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/assign#Polyfill
if (typeof Object.assign != 'function') {
	(function () {
		Object.assign = function (target) {
			'use strict';

			if (target === undefined || target === null) {
				throw new TypeError('Cannot convert undefined or null to object');
			}

			var output = Object(target);
			for (var index = 1; index < arguments.length; index++) {
				var source = arguments[index];
				if (source !== undefined && source !== null) {
					for (var nextKey in source) {
						if (source.hasOwnProperty(nextKey)) {
							output[nextKey] = source[nextKey];
						}
					}
				}
			}
			return output;
		};
	})();
}

exports.default = gulpReactCssUsage;